--[[ include used code ]]
require "table_to_gui_transformations"

--[[ thunderBox actions ]]
local function add_refresh_button(guiFrame, playerindex)
	local myElement = guiFrame.add( { type= "button", name= "tfrRefreshPage", caption= "refresh" } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	myElement.style.fontcolor = { r=0.5,g=0.5,b=1 }
end

local function add_bottom_frame(guiFrame, playerindex, back_button)
	local myFlow = guiFrame.add( { type="flow", direction="horizontal" } )
	myFlow.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	myFlow.style.toppadding = 8
	if back_button then 
		local myElement = myFlow.add( { type= "button", name= "tfrBack", caption="back" } )
		myElement.style.fontcolor = { r=1,g=1,b=0 }
	end
	myFlow.add( { type= "button", name= "tfrClose", caption="close" } )
end

local function create_thunderBox(player, playerindex)
	return player.gui.center.add( { type="frame", name="thunderBox", 
		caption = "Thunder box", style="tms_frame_compact", direction="vertical"} )
end


local function open_or_renew_thunderBox(player, playerindex)
   local myFrame = player.gui.center.thunderBox
 	if myFrame then
      myFrame.destroy();
   end
   return create_thunderBox(player, playerindex)
end

local function show_default_page(player, playerindex)
		local myFrame = open_or_renew_thunderBox(player, playerindex)
		local myElement = myFrame.add( { type= "label", style= "tms_label", caption= tf_VersionString(glob.cfg.version) } )
		myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

		local myFlow = myFrame
		if glob.cfg.developerMode then
			myFlow = myFrame.add( { type="flow", direction="vertical", style="tms_flow_compact" } )
				myFlow.style.toppadding = 2
				myFlow.style.bottompadding = 4
		end

		myElement = myFlow.add( { type= "button", name= "tbnShowGuiConfig", caption= "Gui settings" } )
		myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

		if playerindex <= 1 or (playerindex > 1 and not glob.cfg.safeMultiplayerMode) then		
			myElement = myFlow.add( { type= "button", name= "tbnShowthunderConfig", caption= "Advanced-Gui settings" } )
			myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
		end

		myElement = myFlow.add( { type= "button", name= "tbnPluginSettings", caption= "Plugin settings" } )
		myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

		if glob.cfg.developerMode then
			myFlow = myFrame.add( { type="flow", direction="vertical" } ) --$$ extra spacing

			myElement = myFlow.add( { type= "button", name= "tbnShowDevTools", caption= "developer tools" } )
			myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
		end

		add_bottom_frame(myFrame, playerindex, false)
		glob.thunderFrame_page[playerindex] = 1
end

local function show_guiOptions_page(player, playerindex)
	local myFrame = open_or_renew_thunderBox(player, playerindex)
	local myElement = myFrame.add( { type= "label", style= "tms_label_title", caption= "-->Gui options" } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

	--$$ get selected config
	local selectedConfig = ""
	local viewPlayerSettings = false
	local cfg = glob.playerCfg[playerindex]
	if cfg and ( cfg.viewPlayerSettings or (glob.cfg.safeMultiplayerMode and playerindex > 1 ))then
		viewPlayerSettings = true
		cfg.viewPlayerSettings = true
		selectedConfig = "player "..tostring(playerindex)
	else
		cfg = glob.cfg
		selectedConfig = "global"
	end 

	local myCompactFlow = myFrame.add( { type="flow", direction="vertical", style="tms_flow_compact" } )

	local myHorizontalFlow = myCompactFlow.add( { type= "flow", direction= "horizontal", style= "tms_flow_compact" } )
	myHorizontalFlow.style.minimalwidth = thunderBox_minimalWidth
		myElement = myHorizontalFlow.add( { type= "button", name= "tbnSetsPlyr", style= "tms_button", caption= "player" } )
	if playerindex <= 1 or not glob.cfg.safeMultiplayerMode then
		myElement = myHorizontalFlow.add( { type= "button", name= "tbnSetsGlob", style= "tms_button", caption= "global" } )
	else 
		myElement = myHorizontalFlow.add( { type= "button", style= "tms_buttonInactive", caption= "global" } )
	end

	local mySettingsFlow = myCompactFlow.add( { type= "frame", direction= "vertical", style="tms_frameAlpha" } )
	mySettingsFlow.style.bottompadding = 10

	local myElement = mySettingsFlow.add( {type= "label", style= "tms_label_title",
			caption= string.format( "Settings from: %q", selectedConfig ) } )
	myElement.style.minimalwidth = thunderBox_minimalWidth
	myElement.style.fontcolor = { r=0.7,g=0.7,b=1}


	local myCompactFlow = mySettingsFlow.add( { type="flow", direction="vertical", style="tms_flow_compact" } )
		myCompactFlow.style.toppadding = 2
		myCompactFlow.style.bottompadding = 4
		myCompactFlow.style.leftpadding = 10

	local myHorizontalFlow = myCompactFlow.add( { type= "flow", direction= "horizontal" } )
	myHorizontalFlow.style.minimalwidth = glob.cfg.thunderBox_minimalWidth-10
	myElement = myHorizontalFlow.add( { type= "button", name= "tbnTglClockEnabled", style= "tms_button", caption= "toggle" } )
	myElement = myHorizontalFlow.add( {type= "label", style= "tms_label",
			caption= string.format( "show clock = %s", tostring(cfg.showClock) ) } )


	myHorizontalFlow = myCompactFlow.add( { type= "flow", direction= "horizontal" } )
	myHorizontalFlow.style.minimalwidth = glob.cfg.thunderBox_minimalWidth-10
	myElement = myHorizontalFlow.add( { type= "button", name= "tbnTglSpeedBtnEnabled", style= "tms_button", caption= "toggle" } )
	myElement = myHorizontalFlow.add( { type= "label", style= "tms_label",
			caption= string.format( "show speedButton = %s", tostring(cfg.showSpeedButton) ) } )

	if not viewPlayerSettings then
		myHorizontalFlow = mySettingsFlow.add( { type= "flow", style="tms_flow_compact", direction= "horizontal" } )
		myHorizontalFlow.style.minimalwidth = glob.cfg.thunderBox_minimalWidth-10
		myHorizontalFlow.style.leftpadding = 10
		myElement = myHorizontalFlow.add( { type= "button", name= "tbnTglRemLeftOfSuper", style= "tms_button", caption= "toggle" } )
		myElement.style.fontcolor = { r=1,g=0,b=0 }
		myHorizontalFlow = myHorizontalFlow.add( { type= "flow", direction= "vertical", style="tms_flow_compact" } )
			myElement = myHorizontalFlow.add( {type= "label", style= "tms_label",
					caption= string.format( "removeElementsLeftOfSuperGui = %s", tostring(glob.cfg.removeElementsLeftOfSuperGui) ) } )
			myElement = myHorizontalFlow.add( { type= "label", style= "tms_label", caption= "( can cause crashes in other mods )" } )
			myElement.style.fontcolor = { r=1,g=0,b=0 }
	end

	if cfg.showSettingsButton then
		myElement = mySettingsFlow.add( { type= "button", name= "tbnShowRemoveThunderPage", caption= "remove settings+ button" } )
		myElement.style.fontcolor = { r=1,g=0.5,b=0 }
		myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	else
		myElement = mySettingsFlow.add( { type= "button", name= "tbnToggleThunderButton", caption= "show settings+ button" } )
		myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	end

	add_bottom_frame(myFrame, playerindex, true)
	glob.thunderFrame_page[playerindex] = 6
end

local function show_removeThunder_page(player, playerindex)
	local myFrame = open_or_renew_thunderBox(player, playerindex)

	--$$ get selected config
	local selectedConfig = ""
	local viewPlayerSettings = false
	local cfg = glob.playerCfg[playerindex]
	if cfg and ( cfg.viewPlayerSettings or (glob.cfg.safeMultiplayerMode and playerindex > 1 ))then
		viewPlayerSettings = true
		cfg.viewPlayerSettings = true
		selectedConfig = "player "..tostring(playerindex)
	else
		cfg = glob.cfg
		selectedConfig = "global"
	end 

	local myElement = myFrame.add( { type= "label", style= "tms_label_title", caption= "-->Gui options-->Remove settings+ button: " } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

	myElement = myFrame.add( { type= "label", style= "tms_label",
			caption= "Are you really sure?" } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	myElement.style.font= "default-frame"
	myElement.style.fontcolor = { r=1,g=0.25,b=0 }
	myElement = myFrame.add( { type= "label", style= "tms_label",
			caption= string.format("Remove the settings button from %s?", selectedConfig) } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	myElement.style.font= "default-frame"
	myElement.style.fontcolor = { r=1,g=0.0,b=0 }

	if viewPlayerSettings then
		myElement = myFrame.add( { type= "label", style= "tms_label",
				caption= " 4 ways to bring the button back:" } )
		myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

		myElement = myFrame.add( { type= "label", style= "tms_label",
				caption= "-> changing global settings at a different player" } )
	else
		myElement = myFrame.add( { type= "label", style= "tms_label",
				caption= " 3 ways to bring the button back:" } )
	end
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

	myElement = myFrame.add( { type= "label", style= "tms_label",
			caption= "-> changing/reseting settings before closing settings" } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	myElement = myFrame.add( { type= "label", style= "tms_label",
			caption= "-> use a different version of the ThunderGui mod" } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	myElement = myFrame.add( { type= "label", style= "tms_label",
			caption= "-> hacking" } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	myElement.style.bottompadding = 10

	myElement = myFrame.add( { type= "button", name= "tbnToggleThunderButton", caption= "Yes, remove settings+ button!!" } )
	myElement.style.fontcolor = { r=1,g=0,b=0}
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

	add_bottom_frame(myFrame, playerindex, true)
	glob.thunderFrame_page[playerindex] = 7
end

local function show_ThunderModOptions_page(player, playerindex)
	local myFrame = open_or_renew_thunderBox(player, playerindex)
	local myElement = myFrame.add( { type= "label", style= "tms_label_title", caption= "-->Mod options: (global)" } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

	local btnWidth = 42
	local myFlow = myFrame.add( { type= "flow", direction= "horizontal" } )
	myFlow.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	local myCompactFlow = myFlow.add( { type= "flow", direction= "horizontal", style= "tms_flow_compact" } )
	myElement = myCompactFlow.add( { type= "button", name= "tbnTUpInterval", style= "tms_button", caption= "up" } )
	myElement.style.minimalwidth = btnWidth
	myElement = myCompactFlow.add( { type= "button", name= "tbnTDnInterval", style= "tms_button", caption= "dn" } )
	myElement.style.minimalwidth = btnWidth
	myElement = myFlow.add( {type= "label", style= "tms_label",
			caption= string.format( "update_interval = %d", glob.cfg.update_interval ) } )

--[[	
	myFlow = myFrame.add( { type= "flow", direction= "horizontal" } )
	myFlow.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	local myCompactFlow = myFlow.add( { type= "flow", direction= "horizontal", style= "tms_flow_compact" } )
	myElement = myCompactFlow.add( { type= "button", name= "tbnTUpMinuteR", style= "tms_button", caption= "up" } )
	myElement.style.minimalwidth = btnWidth
	myElement = myCompactFlow.add( { type= "button", name= "tbnTDnMinuteR", style= "tms_button", caption= "dn" } )
	myElement.style.minimalwidth = btnWidth
	myElement = myFlow.add( {type= "label", style= "tms_label",
			caption= string.format( "clock_gt_minuteRounding = %d", glob.cfg.clock_gt_minuteRounding_Value ) } )
--]]

	myFlow = myFrame.add( { type= "flow", direction= "horizontal" } )
	myFlow.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	local myCompactFlow = myFlow.add( { type= "flow", direction= "horizontal", style= "tms_flow_compact" } )
	myElement = myCompactFlow.add( { type= "button", name= "tbnTUpTBoxW", style= "tms_button", caption= "up" } )
	myElement.style.minimalwidth = btnWidth
	myElement = myCompactFlow.add( { type= "button", name= "tbnTDnTBoxW", style= "tms_button", caption= "dn" } )
	myElement.style.minimalwidth = btnWidth
	myElement = myFlow.add( {type= "label", style= "tms_label",
			caption= string.format( "thunderBox_minimalWidth = %d", glob.cfg.thunderBox_minimalWidth ) } )
	
	myFlow = myFrame.add( { type= "flow", direction= "horizontal" } )
	myFlow.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	myElement = myFlow.add( { type= "button", name= "tbnTglRemoveOnSave", style= "tms_button", caption= "toggle" } )
	myElement = myFlow.add( { type= "label", style= "tms_label",
			caption= string.format( "removeGui_onSave = %s", tostring(glob.cfg.removeGui_onSave) ) } )

	myFlow = myFrame.add( { type= "flow", direction= "horizontal" } )
	myFlow.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	myElement = myFlow.add( { type= "button", name= "tbnTglDevMode", style= "tms_button", caption= "toggle" } )
	myElement.style.fontcolor = { r=1,g=0.5,b=0 }
	myElement = myFlow.add( {type= "label", style= "tms_label",
			caption= string.format( "developerMode = %s", tostring(glob.cfg.developerMode) ) } )

	myFlow = myFrame.add( { type= "flow", direction= "horizontal" } )
	myFlow.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	myElement = myFlow.add( { type= "button", name= "tbnTglSafeMode", style= "tms_button", caption= "toggle" } )
	myElement.style.fontcolor = { r=1,g=0,b=0 }
	myElement = myFlow.add( {type= "label", style= "tms_label",
			caption= string.format( "safeMultiplayerMode = %s", tostring(glob.cfg.safeMultiplayerMode) ) } )

	myElement = myFrame.add( { type= "button", name= "tbnResetCfg", caption= "reset ThunderGui configuration" } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	myElement.style.fontcolor = { r=1,g=0.5,b=0}

	add_bottom_frame(myFrame, playerindex, true)
	glob.thunderFrame_page[playerindex] = 10
end

local function show_pluginSettings_page(player, playerindex)
	local myFrame = open_or_renew_thunderBox(player, playerindex)
	local myElement = myFrame.add( { type= "label", style= "tms_label_title", caption= "-->Plugin settings: " } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

	if glob.plugins and #glob.plugins > 0 then
		local myFlow = myFrame.add( { type= "flow", direction= "vertical" } )
		myFlow.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
		local missingSettingsButton = false

		for key, modDataTable in pairs(glob.plugins) do
			if modDataTable.settingsBtn_name then
				myElement = myFlow.add( { type= "button", name= modDataTable.settingsBtn_name, caption= modDataTable.name } )
				myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth				
			else
				missingSettingsButton = true
			end
		end

		if missingSettingsButton then
			local myFlow = myFrame.add( { type= "flow", direction= "vertical" } )
			myFlow.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
			myElement = myFlow.add( { type= "label", style= "tms_label", caption= "plugins without settings:" } )
			myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth				

			for key, modDataTable in pairs(glob.plugins) do
				if not modDataTable.settingsBtn_name then
					myElement = myFlow.add( { type= "label", caption= modDataTable.name } )
					myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth				
					myElement.style.leftpadding = 20
				end
			end
		end

		if playerindex <= 1 or (playerindex > 1 and not glob.cfg.safeMultiplayerMode) then		
			myElement = myFrame.add( { type= "button", name= "tbnShowAdvPlugins", caption= "remove plugins" } )
			myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
			myElement.style.fontcolor = { r=1,g=0.5,b=0}
		end
	else
		myFrame.add( { type= "label", style= "tms_label", caption= "no plugins installed" } )
	end

	add_bottom_frame(myFrame, playerindex, true)
	glob.thunderFrame_page[playerindex] = 11
end

local function show_Advancedplugin_page(player, playerindex)
	local myFrame = open_or_renew_thunderBox(player, playerindex)
	local myElement = myFrame.add( { type= "label", style= "tms_label_title", caption= "-->Plugin settings:-->remove:" } )
	myElement.style.fontcolor = { r=1,g=0.5,b=0}
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

	local myElement = myFrame.add( { type= "label", style= "tms_label_fat", caption= "Warning:" } )
	myElement.style.fontcolor = { r=1,g=0.0,b=0}
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	local myElement = myFrame.add( { type= "label", style= "tms_label_fat", caption= "A removed plugin can't be restored!" } )
	myElement.style.fontcolor = { r=1,g=0.0,b=0}
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

	if glob.plugins and #glob.plugins > 0 then
		local myFlow = myFrame.add( { type= "flow", direction= "vertical" } )
		myFlow.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

		for key, modDataTable in pairs(glob.plugins) do
			myElement = myFlow.add( { type= "button", name= string.format( "tbnRemPI_%s", modDataTable.name ), caption= modDataTable.name } )
			myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth				
		end

		--$$ remove plugins
		myElement = myFrame.add( { type= "button", name= "tbnRemovePlugins", caption= "remove all plugins" } )
		myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
		myElement.style.fontcolor = { r=1,g=0,b=0}
	end

	add_bottom_frame(myFrame, playerindex, true)
	glob.thunderFrame_page[playerindex] = 12
end

local function show_devGlobTables_page(player, playerindex)
	local myFrame = open_or_renew_thunderBox(player, playerindex)
	local myElement = myFrame.add( { type= "label", style= "tms_label_title", caption= "-->Dev-->glob tables:" } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

	transform_keys_to_labels(glob, "glob", myFrame)
	transform_keysvaluepairs_to_labels(glob.cfg, "glob.cfg", myFrame)

	add_refresh_button(myFrame, playerindex)
	add_bottom_frame(myFrame, playerindex, true)
	glob.thunderFrame_page[playerindex] = 101
end

local function show_devTools_page(player, playerindex)
	local myFrame = open_or_renew_thunderBox(player, playerindex)
	local myElement = myFrame.add( { type= "label", style= "tms_label_title", caption= "-->Developer tools: " } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	
	local myFlow = myFrame.add( { type="flow", direction="vertical" --[[, style="tms_flow_compact"--]] } )
		myFlow.style.bottompadding = 4

	myFlow = myFrame.add( { type="flow", direction="vertical", style="tms_flow_compact" } )

	myElement = myFlow.add( { type= "button", name= "tbnShowGlobTables", caption= "glob tables" } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth

	myElement = myFlow.add( { type= "button", name= "tbnDestroySuperGui", caption= "destroy superTopGui" } )
	myElement.style.minimalwidth = glob.cfg.thunderBox_minimalWidth
	myElement.style.fontcolor = { r=0.2,g=1,b=0.2 }

	add_bottom_frame(myFrame, playerindex, true)
	glob.thunderFrame_page[playerindex] = 100
end

local function thunderBox_pageUpdate(player, playerindex)
	thunderBox_remove(player, playerindex)

	if not glob.thunderFrame_page then
		glob.thunderFrame_page = {}
		glob.thunderFrame_page[playerindex] = 0
		return
	elseif not glob.thunderFrame_page[playerindex] or glob.thunderFrame_page[playerindex] <= 0 then 
		glob.thunderFrame_page[playerindex] = 0
		return
	elseif glob.thunderFrame_page[playerindex] == 1 then 
		show_default_page(player, playerindex)
	elseif glob.thunderFrame_page[playerindex] == 6 then 
		show_guiOptions_page(player, playerindex)
	elseif glob.thunderFrame_page[playerindex] == 7 then 
		show_removeThunder_page(player, playerindex)
	elseif glob.thunderFrame_page[playerindex] == 10 then
		show_ThunderModOptions_page(player, playerindex)
	elseif glob.thunderFrame_page[playerindex] == 11 then
		show_pluginSettings_page(player, playerindex)
	elseif glob.thunderFrame_page[playerindex] == 12 then
		show_Advancedplugin_page(player, playerindex)

	elseif glob.thunderFrame_page[playerindex] == 100 then
		show_devTools_page(player, playerindex)
	elseif glob.thunderFrame_page[playerindex] == 101 then 
		show_devGlobTables_page(player, playerindex)
	end
end

local function thunderButton_changeCaption(player, caption)
	stg_setActionElementCaption(player, "tbnThunder", caption)
end

local function thunderBox_openClose(player, playerindex)
	if not player.gui.center.thunderBox then
		show_default_page(player, playerindex)
		thunderButton_changeCaption(player, "Settings -")
	else
		player.gui.center.thunderBox.destroy()
		glob.thunderFrame_page[playerindex] = 0
		thunderButton_changeCaption(player, "Settings +")
	end
end

function thunderBox_remove(player, playerindex)
	if player.gui.center.thunderBox then
		player.gui.center.thunderBox.destroy()
	end
end

function thunderBox_close(player, playerindex)
	thunderBox_remove(player, playerindex)
	if glob.thunderFrame_page then
		glob.thunderFrame_page[playerindex] = 0
	end
	thunderButton_changeCaption(player, "Settings +")
end

--[[ PUBLIC stuff ]]
function tf_thunderButton_remove(player, playerindex)
	stg_destroyActionElement(player, "tbnThunder")
	thunderBox_remove(player, playerindex)
end

function tf_thunderButton_create(player, playerindex)
	--if not glob.cfg.showSettingsButton then return end
	--if ( playerindex == 1 ) or ( not glob.cfg.safeMultiplayerMode and playerindex > 1 ) then
	--$$ button
	local cfg = glob.playerCfg[playerindex]
	if not cfg or (cfg and cfg.showSettingsButton) then
		if player.gui.center.thunderBox then
			stg_createActionButton(player, "tbnThunder", "Settings -")
		else
			stg_createActionButton(player, "tbnThunder", "Settings +")
		end
	end
	--$$ frame
	thunderBox_pageUpdate(player, playerindex)
	--end
end

function tf_thunderButton_onguiclick(event)
	local playerindex = event.playerindex
	local player = game.players[playerindex]

	--$$ regular used buttons
	if event.element.name == "tbnThunder" then
		thunderBox_openClose(player, playerindex) --show_default_page()
	elseif event.element.name == "tfrClose" then
		thunderBox_close(player, playerindex)
	elseif event.element.name == "tfrBack" then 
		--thunderBox_close(player, playerindex)
		if glob.thunderFrame_page[playerindex] and glob.thunderFrame_page[playerindex] == 7 then --$$ from remove settings+ page
			show_guiOptions_page(player, playerindex)
		elseif glob.thunderFrame_page[playerindex] and glob.thunderFrame_page[playerindex] == 12 then --$$ from remove plugins page
			show_pluginSettings_page(player, playerindex)
		elseif glob.thunderFrame_page[playerindex] and glob.thunderFrame_page[playerindex] > 100 then--$$ from dev-glob page
			show_devTools_page(player, playerindex)
		else
			show_default_page(player, playerindex)
		end
	elseif event.element.name == "tfrRefreshPage" then
		thunderBox_pageUpdate(player, playerindex)


	--$$ pages
	elseif event.element.name == "tbnShowGuiConfig" then
		show_guiOptions_page(player, playerindex)
	elseif event.element.name == "tbnShowthunderConfig" then
		show_ThunderModOptions_page(player, playerindex)
	elseif event.element.name == "tbnShowRemoveThunderPage" then
		show_removeThunder_page(player, playerindex)
	elseif event.element.name == "tbnPluginSettings" then
		show_pluginSettings_page(player, playerindex)
	elseif event.element.name == "tbnShowAdvPlugins" then
		show_Advancedplugin_page(player, playerindex)
	elseif event.element.name == "tbnShowDevTools" then
		show_devTools_page(player, playerindex)
	elseif event.element.name == "tbnShowGlobTables" then
		show_devGlobTables_page(player, playerindex)

	--$$ Select Settings table
	elseif event.element.name == "tbnSetsPlyr" then
		local cfg = glob.playerCfg[playerindex]
		if cfg and not cfg.viewPlayerSettings then
			cfg.viewPlayerSettings = true
		end 
		thunderBox_pageUpdate(player, playerindex)

	elseif event.element.name == "tbnSetsGlob" then
		local cfg = glob.playerCfg[playerindex]
		if cfg and cfg.viewPlayerSettings then
			cfg.viewPlayerSettings = false
		end 
		thunderBox_pageUpdate(player, playerindex)

	--$$ toggle glob.cfg or glob.playerCfg and refresh gui and/or page 
	elseif event.element.name == "tbnToggleThunderButton" then
		local cfg = glob.playerCfg[event.playerindex]
		local updatePlayerOnly = true
		if not cfg or not cfg.viewPlayerSettings then
			cfg = glob.cfg
			updatePlayerOnly = false
		end 

		cfg.showSettingsButton = not cfg.showSettingsButton
		--thunderBox_close(player, playerindex)
		show_guiOptions_page(player, playerindex)
		--tf_thunderButton_remove(player, playerindex)
		if updatePlayerOnly then
			stg_update(player)
		else
			for locPlayerindex, locPlayer in pairs(game.players) do
				glob.playerCfg[locPlayerindex].showSettingsButton = cfg.showSettingsButton
				stg_update(locPlayer)
			end
		end
	elseif event.element.name == "tbnTglClockEnabled" then
		local cfg = glob.playerCfg[event.playerindex]
		local updatePlayerOnly = true
		if not cfg or not cfg.viewPlayerSettings then
			cfg = glob.cfg
			updatePlayerOnly = false
		end 

		cfg.showClock = not cfg.showClock
		thunderBox_pageUpdate(player, playerindex)
		if updatePlayerOnly then
			if cfg.showClock then
				stg_update(player)
			else
				tf_clockDisplay_remove(player, playerindex)
			end
		else
			if cfg.showClock then
				for locPlayerindex, locPlayer in pairs(game.players) do
					glob.playerCfg[locPlayerindex].showClock = cfg.showClock
					stg_update(locPlayer)
				end
			else
				for locPlayerindex, locPlayer in pairs(game.players) do
					glob.playerCfg[locPlayerindex].showClock = cfg.showClock
					tf_clockDisplay_remove(locPlayer, locPlayerindex)
				end
			end
		end
	elseif event.element.name == "tbnTglSpeedBtnEnabled" then
		local cfg = glob.playerCfg[event.playerindex]
		local updatePlayerOnly = true
		if not cfg or not cfg.viewPlayerSettings then
			cfg = glob.cfg
			updatePlayerOnly = false
		end 

		cfg.showSpeedButton = not cfg.showSpeedButton
		thunderBox_pageUpdate(player, playerindex)
		if updatePlayerOnly then
			if cfg.showSpeedButton then
				stg_update(player)
			else
				tf_speedButton_remove(player, playerindex)
			end
		else
			if cfg.showSpeedButton then
				for locPlayerindex, locPlayer in pairs(game.players) do
					glob.playerCfg[locPlayerindex].showSpeedButton = cfg.showSpeedButton
					stg_update(locPlayer)
				end
			else
				for locPlayerindex, locPlayer in pairs(game.players) do
					glob.playerCfg[locPlayerindex].showSpeedButton = cfg.showSpeedButton
					tf_speedButton_remove(locPlayer, locPlayerindex)
				end
			end
		end
	elseif event.element.name == "tbnTglSafeMode" then
		glob.cfg.safeMultiplayerMode = not glob.cfg.safeMultiplayerMode
		--stg_updateAll()
		--thunderBox_pageUpdate(player, playerindex)
		for locPlayerindex, locPlayer in pairs(game.players) do
			thunderBox_pageUpdate(locPlayer, locPlayerindex)
		end
	elseif event.element.name == "tbnTglDevMode" then
		glob.cfg.developerMode = not glob.cfg.developerMode
		thunderBox_pageUpdate(player, playerindex)
	elseif event.element.name == "tbnTglRemoveOnSave" then
		glob.cfg.removeGui_onSave = not glob.cfg.removeGui_onSave
		thunderBox_pageUpdate(player, playerindex)
	elseif event.element.name == "tbnTglRemLeftOfSuper" then
		glob.cfg.removeElementsLeftOfSuperGui = not glob.cfg.removeElementsLeftOfSuperGui
		if glob.cfg.removeElementsLeftOfSuperGui then
			stg_updateAll()
		end
		thunderBox_pageUpdate(player, playerindex)


	--$$ TUps and TDowns
	elseif event.element.name == "tbnTUpInterval" then
		if glob.cfg.update_interval < 3600 then -- one minute
			glob.cfg.update_interval = glob.cfg.update_interval + 10
			thunderBox_pageUpdate(player, playerindex)
		end
	elseif event.element.name == "tbnTDnInterval" then
		if glob.cfg.update_interval > 10 then
			glob.cfg.update_interval = glob.cfg.update_interval - 10
			thunderBox_pageUpdate(player, playerindex)
		end

--[[
	elseif event.element.name == "tbnTUpMinuteR" then
		if glob.cfg.clock_gt_minuteRounding_Value < 30 then
			glob.cfg.clock_gt_minuteRounding_Value = glob.cfg.clock_gt_minuteRounding_Value + 1
			thunderBox_pageUpdate(player, playerindex)
		end
	elseif event.element.name == "tbnTDnMinuteR" then
		if glob.cfg.clock_gt_minuteRounding_Value > 1 then
			glob.cfg.clock_gt_minuteRounding_Value = glob.cfg.clock_gt_minuteRounding_Value - 1
			thunderBox_pageUpdate(player, playerindex)
		end
--]]

	elseif event.element.name == "tbnTUpTBoxW" then
		if glob.cfg.thunderBox_minimalWidth < 800 then -- one minute
			glob.cfg.thunderBox_minimalWidth = glob.cfg.thunderBox_minimalWidth + 20
			thunderBox_pageUpdate(player, playerindex)
		end
	elseif event.element.name == "tbnTDnTBoxW" then
		if glob.cfg.thunderBox_minimalWidth > 200 then
			glob.cfg.thunderBox_minimalWidth = glob.cfg.thunderBox_minimalWidth - 20
			thunderBox_pageUpdate(player, playerindex)
		end


	--$$ glob.cfg reset
	elseif event.element.name == "tbnResetCfg" then
		tf_reset_cfg(glob.cfg)
		tf_reset_allPlayerCfgs()
		stg_updateAll()
		
		thunderBox_pageUpdate(player, playerindex)

	elseif event.element.name == "tbnRemovePlugins" then
		for locPlayerindex, locPlayer in pairs(game.players) do
			ctrl_remove_gui(locPlayer, locPlayerindex)
		end
		tf_remove_allPlugins()
		thunderBox_pageUpdate(player, playerindex)
	
	elseif event.element.name == "tbnDestroySuperGui" then
		stg_update(player)--[[
		--$$ destroy like a 'hacker' would, and not call stg_update().
		if player.gui.top.superTopGui then
			player.gui.top.superTopGui.destroy()
		end--]]
		thunderBox_pageUpdate(player, playerindex)

	else
		local pos = string.find(event.element.name, "tbnRemPI_")
		if pos == nil or pos ~= 1 then
			--$$ did not recognize button
			return false
		end
		pluginName = string.sub(event.element.name, 10)
		if pluginName == nil then return true end --$$ did recognize button, but shit hits the fan

		for locPlayerindex, locPlayer in pairs(game.players) do
			ctrl_remove_gui(locPlayer, locPlayerindex)
		end
		tf_remove_plugin(pluginName)
		thunderBox_pageUpdate(player, playerindex)
	end
	return true
end
